@@ -35,10 +35,12 @@ of the License or (at your option) any later version.
 #include <cstring>
 #include "graphite2/Segment.h"
 #include "inc/Code.h"
-#include "inc/Machine.h"
-#include "inc/Silf.h"
 #include "inc/Face.h"
+#include "inc/GlyphFace.h"
+#include "inc/GlyphCache.h"
+#include "inc/Machine.h"
 #include "inc/Rule.h"
+#include "inc/Silf.h"
 
 #include <cstdio>
 
@@ -139,7 +141,7 @@ inline Machine::Code::decoder::decoder(const limits & lims, Code &code) throw()
 
 Machine::Code::Code(bool is_constraint, const byte * bytecode_begin, const byte * const bytecode_end,
            uint8 pre_context, uint16 rule_length, const Silf & silf, const Face & face)
- :  _code(0), _data_size(0), _instr_count(0), _status(loaded),
+ :  _code(0), _data(0), _data_size(0), _instr_count(0), _max_ref(0), _status(loaded),
     _constraint(is_constraint), _modify(false), _delete(false), _own(true)
 {
     assert(bytecode_begin != 0);
@@ -168,7 +170,7 @@ Machine::Code::Code(bool is_constraint, const byte * bytecode_begin, const byte
         pre_context,
         rule_length,
         silf.numClasses(),
-        face.getGlyphFaceCache()->numAttrs(),
+        face.glyphs().numAttrs(),
         face.numFeatures(), 
         {1,1,1,1,1,1,1,1, 
          1,1,1,1,1,1,1,255,
@@ -307,6 +309,7 @@ opcode Machine::Code::decoder::fetch_opcode(const byte * bc)
         case CNTXT_ITEM :
             valid_upto(_max.rule_length, _max.pre_context + int8(bc[0]));
             if (bc + 2 + bc[1] >= _max.bytecode)  failure(jump_past_end);
+            if (_pre_context != 0)                failure(nested_context_item);
             break;
         case ATTR_SET :
         case ATTR_ADD :
@@ -419,7 +422,7 @@ void Machine::Code::decoder::analyse_opcode(const opcode opc, const int8  * arg)
     case PUT_SUBS : 
       _code._modify = true;
       _analysis.set_changed(_analysis.slotref);
-      // no break here;
+      // no break
     case PUT_COPY :
     {
       if (arg[0] != 0) { _analysis.set_changed(_analysis.slotref); _code._modify = true; }
@@ -430,6 +433,7 @@ void Machine::Code::decoder::analyse_opcode(const opcode opc, const int8  * arg)
     }
     case PUSH_ATT_TO_GATTR_OBS : // slotref on 2nd parameter
         if (_code._constraint) return;
+        // no break
     case PUSH_GLYPH_ATTR_OBS :
     case PUSH_SLOT_ATTR :
     case PUSH_GLYPH_METRIC :
@@ -442,6 +446,7 @@ void Machine::Code::decoder::analyse_opcode(const opcode opc, const int8  * arg)
       break;
     case PUSH_ATT_TO_GLYPH_ATTR :
         if (_code._constraint) return;
+        // no break
     case PUSH_GLYPH_ATTR :
       if (arg[2] <= 0 && -arg[2] <= _analysis.slotref - _analysis.contexts[_analysis.slotref].flags.inserted)
         _analysis.set_ref(_analysis.slotref + arg[2] - _analysis.contexts[_analysis.slotref].flags.inserted);
